@@ -16,6 +16,7 @@ from robotframework_ls.impl.protocols import (
     IVariableFound,
     AbstractVariablesCollector,
     VariableKind,
+    AbstractKeywordCollector,
 )
 from robotframework_ls.impl.robot_lsp_constants import (
     OPTION_ROBOT_LINT_VARIABLES,
@@ -23,6 +24,7 @@ from robotframework_ls.impl.robot_lsp_constants import (
     OPTION_ROBOT_LINT_IGNORE_ENVIRONMENT_VARIABLES,
 )
 from robotframework_ls.impl.robot_constants import STDLIBS_LOWER
+import typing
 
 
 log = get_logger(__name__)
@@ -387,7 +389,7 @@ def collect_analysis_errors(initial_completion_context):
                 errors.append(error)
 
             else:
-                new_keywords_found = []
+                new_keywords_found: List[IKeywordFound] = []
                 if len(keywords_found) > 1:
                     # We still can't be sure, it's possible that we found the
                     # same keyword multiple times. Let's check where they're found.
@@ -479,14 +481,6 @@ def collect_analysis_errors(initial_completion_context):
                             )
                             errors.append(error)
 
-                        # We can't do argument analysis because the keyword is
-                        # duplicated and we don't know which one to use.
-                        continue
-
-                if new_keywords_found:
-                    keywords_found = new_keywords_found
-                keyword_found = keywords_found[0]
-
                 from robotframework_ls.impl.robot_lsp_constants import (
                     OPTION_ROBOT_LINT_KEYWORD_CALL_ARGUMENTS,
                 )
@@ -500,59 +494,75 @@ def collect_analysis_errors(initial_completion_context):
                     KeywordArgumentAnalysis,
                 )
 
-                keyword_token = keyword_usage_info.node.get_token(Token.KEYWORD)
-                if keyword_token is not None:
-                    # Ok, we found the keyword, let's check if the arguments are correct.
-                    keyword_argument_analysis = KeywordArgumentAnalysis(
-                        keyword_found.keyword_args
-                    )
+                if new_keywords_found:
+                    keywords_found = new_keywords_found
 
-                    for error in keyword_argument_analysis.collect_keyword_usage_errors(
-                        UsageInfoForKeywordArgumentAnalysis(
-                            keyword_usage_info.node,
-                            keyword_token,
+                # Still do the keyword analysis even if multiple keywords match
+                # See: https://github.com/robocorp/robotframework-lsp/issues/724
+                found_error_in_arg_analysis = False
+                for keyword_found in keywords_found:
+                    if found_error_in_arg_analysis:
+                        break
+
+                    keyword_token = keyword_usage_info.node.get_token(Token.KEYWORD)
+                    if keyword_token is not None:
+                        # Ok, we found the keyword, let's check if the arguments are correct.
+                        keyword_argument_analysis = KeywordArgumentAnalysis(
+                            keyword_found.keyword_args
                         )
-                    ):
-                        errors.append(error)
-                else:
-                    # Not a keyword usage, check for other cases (template/fixtures).
-                    if keyword_usage_info.node.type in (
-                        Token.TEMPLATE,
-                        Token.TEST_TEMPLATE,
-                    ):
-                        # For templates the arguments are actually gotten from the test.
-                        stack = keyword_usage_info.stack
-                        if keyword_usage_info.node.type == Token.TEST_TEMPLATE:
-                            stack = [ast]
-                        for (
-                            template_arguments_node_info
-                        ) in ast_utils.iter_arguments_from_template(
-                            stack, keyword_usage_info.node
-                        ):
-                            keyword_argument_analysis = KeywordArgumentAnalysis(
-                                keyword_found.keyword_args
-                            )
-                            args_tokens = template_arguments_node_info.node.tokens
-                            for (
-                                error
-                            ) in keyword_argument_analysis.collect_keyword_usage_errors(
-                                UsageInfoForKeywordArgumentAnalysis(
-                                    template_arguments_node_info.node,
-                                    args_tokens[-1],
-                                    args_tokens,
-                                )
-                            ):
-                                errors.append(error)
 
-                if keyword_found.is_deprecated():
-                    error = create_error_from_node(
-                        keyword_usage_info.node,
-                        f"Keyword: {keyword_usage_info.name} is deprecated",
-                        tokens=[keyword_usage_info.token],
-                    )
-                    error.severity = DiagnosticSeverity.Hint
-                    error.tags = [DiagnosticTag.Deprecated]
-                    errors.append(error)
+                        for (
+                            error
+                        ) in keyword_argument_analysis.collect_keyword_usage_errors(
+                            UsageInfoForKeywordArgumentAnalysis(
+                                keyword_usage_info.node,
+                                keyword_token,
+                            )
+                        ):
+                            errors.append(error)
+                            found_error_in_arg_analysis = True
+                    else:
+                        # Not a keyword usage, check for other cases (template/fixtures).
+                        if keyword_usage_info.node.type in (
+                            Token.TEMPLATE,
+                            Token.TEST_TEMPLATE,
+                        ):
+                            # For templates the arguments are actually gotten from the test.
+                            stack = keyword_usage_info.stack
+                            if keyword_usage_info.node.type == Token.TEST_TEMPLATE:
+                                stack = [ast]
+                            for (
+                                template_arguments_node_info
+                            ) in ast_utils.iter_arguments_from_template(
+                                stack, keyword_usage_info.node
+                            ):
+                                keyword_argument_analysis = KeywordArgumentAnalysis(
+                                    keyword_found.keyword_args
+                                )
+                                args_tokens = template_arguments_node_info.node.tokens
+                                for (
+                                    error
+                                ) in keyword_argument_analysis.collect_keyword_usage_errors(
+                                    UsageInfoForKeywordArgumentAnalysis(
+                                        template_arguments_node_info.node,
+                                        args_tokens[-1],
+                                        args_tokens,
+                                    )
+                                ):
+                                    errors.append(error)
+                                    found_error_in_arg_analysis = True
+
+                for keyword_found in keywords_found:
+                    if keyword_found.is_deprecated():
+                        error = create_error_from_node(
+                            keyword_usage_info.node,
+                            f"Keyword: {keyword_usage_info.name} is deprecated",
+                            tokens=[keyword_usage_info.token],
+                        )
+                        error.severity = DiagnosticSeverity.Hint
+                        error.tags = [DiagnosticTag.Deprecated]
+                        errors.append(error)
+                        break
 
             if len(errors) >= MAX_ERRORS:
                 # i.e.: Collect at most 100 errors
@@ -560,15 +570,96 @@ def collect_analysis_errors(initial_completion_context):
         except:
             log.exception("Exception collecting errors")
 
+    if len(errors) >= MAX_ERRORS:
+        return errors
+
     for error in _collect_undefined_variables_errors(initial_completion_context):
         errors.append(error)
         if len(errors) >= MAX_ERRORS:
             # i.e.: Collect at most 100 errors
             break
 
+    if len(errors) >= MAX_ERRORS:
+        return errors
+
+    _collect_unused_keyword_errors(initial_completion_context, errors)
+
     return errors
 
 
+class _NoReferencesErrorsKeywordsCollector(AbstractKeywordCollector):
+    def __init__(self, errors):
+        from robocorp_ls_core.lsp import Error
+
+        self.errors: List[Error] = errors
+
+    def accepts(self, keyword_name: str) -> bool:
+        return True
+
+    def on_keyword(self, keyword_found: IKeywordFound):
+        from robocorp_ls_core.lsp import Error
+
+        if len(self.errors) >= MAX_ERRORS:
+            # i.e.: Collect at most 100 errors
+            return
+
+        from robotframework_ls.impl.references import references_for_keyword_found
+
+        completion_context = keyword_found.completion_context
+        assert completion_context
+        references = references_for_keyword_found(
+            completion_context, keyword_found, include_declaration=False
+        )
+        if not references:
+            start = (keyword_found.lineno, keyword_found.col_offset)
+            end = (keyword_found.end_lineno, keyword_found.end_col_offset)
+
+            error = Error(
+                f"The keyword: '{keyword_found.keyword_name}' is not used in the workspace.",
+                start,
+                end,
+                DiagnosticSeverity.Warning,
+            )
+            self.errors.append(error)
+
+
+def _collect_unused_keyword_errors(completion_context: ICompletionContext, errors):
+    from robotframework_ls.impl.robot_lsp_constants import (
+        OPTION_ROBOT_LINT_UNUSED_KEYWORD,
+    )
+    from robotframework_ls.impl.robot_workspace import RobotWorkspace
+
+    config = completion_context.config
+
+    if config is not None and config.get_setting(
+        OPTION_ROBOT_LINT_UNUSED_KEYWORD,
+        bool,
+        False,
+    ):
+
+        from robotframework_ls.impl.collect_keywords import collect_keywords_from_ast
+
+        # The lint process usually does not have workspace indexing turned on,
+        # but it's needed for finding unused references, so, turn it on now.
+
+        workspace: Optional[RobotWorkspace] = typing.cast(
+            Optional[RobotWorkspace], completion_context.workspace
+        )
+        if not workspace:
+            log.critical("Not analyzing unused keywords because workspace is None.")
+            return
+
+        workspace_indexer = workspace.workspace_indexer
+        if workspace_indexer is None:
+            workspace.setup_workspace_indexer()
+            workspace_indexer = workspace.workspace_indexer
+            assert workspace_indexer is not None
+
+        ast = completion_context.get_ast()
+        collector = _NoReferencesErrorsKeywordsCollector(errors)
+        collect_keywords_from_ast(ast, completion_context, collector)
+
+
 @lru_cache(maxsize=1000)
 def _skip_variable_analysis(normalized_variable_name):
     from robotframework_ls.impl.variable_resolve import is_number_var
