@@ -10,6 +10,7 @@ from robotframework_ls.impl.protocols import (
     IKeywordArg,
     LibraryDependencyInfo,
     AbstractKeywordCollector,
+    INode,
 )
 from typing import Sequence, List, Dict, Optional, Iterator, Any
 from robotframework_ls.impl.text_utilities import build_keyword_docs_with_signature
@@ -57,6 +58,10 @@ class _KeywordFoundFromAst(object):
     def keyword_name(self):
         return self._keyword_name
 
+    @property
+    def keyword_ast(self) -> Optional[INode]:
+        return self._keyword_node
+
     @property
     def keyword_args(self) -> Sequence[IKeywordArg]:
         return self._keyword_args
@@ -186,6 +191,10 @@ class _KeywordFoundFromLibrary(object):
     def keyword_name(self):
         return self._keyword_name
 
+    @property
+    def keyword_ast(self) -> Optional[INode]:
+        return None
+
     @property
     def keyword_args(self) -> Sequence[IKeywordArg]:
         return self._keyword_args
@@ -274,8 +283,8 @@ class _KeywordFoundFromLibrary(object):
         _: IKeywordFound = check_implements(self)
 
 
-def _collect_completions_from_ast(
-    ast, completion_context: ICompletionContext, collector
+def collect_keywords_from_ast(
+    ast, completion_context: ICompletionContext, collector: IKeywordCollector
 ):
     from robotframework_ls.impl import ast_utils
     from robocorp_ls_core.lsp import CompletionItemKind
@@ -313,7 +322,7 @@ def _collect_current_doc_keywords(
     # Get keywords defined in the file itself
 
     ast = completion_context.get_ast()
-    _collect_completions_from_ast(ast, completion_context, collector)
+    collect_keywords_from_ast(ast, completion_context, collector)
 
 
 def _collect_libraries_keywords(
