@@ -1,3 +1,6 @@
+import json
+import tempfile
+from robocorp_ls_core.command_dispatcher import _CommandDispatcher
 from robocorp_ls_core.python_ls import PythonLanguageServer
 from robocorp_ls_core.basic import (
     overrides,
@@ -6,9 +9,12 @@ from robocorp_ls_core.basic import (
 )
 import os
 import time
-from robotframework_ls.constants import DEFAULT_COMPLETIONS_TIMEOUT
+from robotframework_ls.constants import (
+    DEFAULT_COMPLETIONS_TIMEOUT,
+    DEFAULT_COLLECT_DOCS_TIMEOUT,
+)
 from robocorp_ls_core.robotframework_log import get_logger
-from typing import Any, Optional, Dict, Sequence, Generator, Set, ContextManager
+from typing import Any, Optional, Dict, Sequence, Set, ContextManager, Union
 from robocorp_ls_core.protocols import (
     IConfig,
     IWorkspace,
@@ -17,6 +23,7 @@ from robocorp_ls_core.protocols import (
     IMonitor,
     IEndPoint,
     IProgressReporter,
+    ActionResultDict,
 )
 from pathlib import Path
 from robotframework_ls.ep_providers import (
@@ -42,11 +49,14 @@ from robocorp_ls_core.lsp import (
 )
 from robotframework_ls.commands import (
     ROBOT_GET_RFLS_HOME_DIR,
+    ROBOT_OPEN_FLOW_EXPLORER_INTERNAL,
     ROBOT_START_INDEXING_INTERNAL,
     ROBOT_WAIT_FULL_TEST_COLLECTION_INTERNAL,
     ROBOT_RF_INFO_INTERNAL,
     ROBOT_LINT_WORKSPACE,
     ROBOT_LINT_EXPLORER,
+    ROBOT_GENERATE_FLOW_EXPLORER_MODEL,
+    ROBOT_COLLECT_ROBOT_DOCUMENTATION,
 )
 from robocorp_ls_core.jsonrpc.exceptions import JsonRpcException
 import weakref
@@ -278,8 +288,6 @@ class _LintManager(object):
             self.schedule_lint(next_uri_to_lint, True, 0.0)
 
 
-from robocorp_ls_core.command_dispatcher import _CommandDispatcher
-
 command_dispatcher = _CommandDispatcher()
 
 
@@ -477,7 +485,8 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
         server_capabilities = {
             "codeActionProvider": False,
             "codeLensProvider": {"resolveProvider": True},
-            "completionProvider": {"resolveProvider": True},  # Docs are lazily computed
+            # Docs are lazily computed
+            "completionProvider": {"resolveProvider": True},
             "documentFormattingProvider": True,
             "documentHighlightProvider": True,
             "documentRangeFormattingProvider": False,
@@ -545,6 +554,44 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
         self._pm.load_plugins_from(Path(directory))
         return True
 
+    @command_dispatcher(ROBOT_COLLECT_ROBOT_DOCUMENTATION)
+    def _collect_robot_documentation(
+        self, opts
+    ) -> Union[ActionResultDict, "partial[Any]"]:
+        uri = opts.get("uri")
+        if not uri:
+            return dict(success=False, message="uri not provided", result=None)
+
+        # We can either accept uri/library name or uri/line/col.
+        # If both are provided the library_name is preferred.
+        library_name = opts.get("library_name")
+        line = opts.get("line")
+        col = opts.get("col")
+        if not library_name:
+            if line is None or col is None:
+                return dict(
+                    success=False,
+                    message="Either library_name or line/col must be provided.",
+                    result=None,
+                )
+
+        rf_api_client = self._server_manager.get_others_api_client(uri)
+        if rf_api_client is not None:
+            func = partial(
+                self._threaded_api_request_no_doc,
+                rf_api_client,
+                "request_collect_robot_documentation",
+                __timeout__=DEFAULT_COLLECT_DOCS_TIMEOUT,
+                doc_uri=uri,
+                library_name=library_name,
+                line=line,
+                col=col,
+            )
+            func = require_monitor(func)
+            return func
+
+        return dict(success=False, message="no api available", result=None)
+
     @command_dispatcher(ROBOT_START_INDEXING_INTERNAL)
     def _start_indexing(self, *arguments):
         self._server_manager.get_regular_rf_api_client("")
@@ -706,6 +753,98 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
         log.info("Unable to get RF info (no api available).")
         return None
 
+    @command_dispatcher(ROBOT_GENERATE_FLOW_EXPLORER_MODEL)
+    def _generate_flow_explorer_model(self, opts: Dict[str, Any]):
+        """
+        :param opts:
+            A dictionary with the options to generate the flow explorer model.
+            Options available:
+                - uri: target uri mapping to the robot or directory with multiple
+                  robots from where the model should be generated.
+
+        """
+        uri = opts["uri"]
+
+        return self.async_api_forward("request_flow_explorer_model", "api", doc_uri=uri)
+
+    @command_dispatcher(ROBOT_OPEN_FLOW_EXPLORER_INTERNAL)
+    def _open_flow_explorer(self, opts) -> ActionResultDict:
+        """
+        :param dict opts:
+            Keys:
+                "currentFileUri": uri for target (may also be folder).
+                "htmlBundleFolderPath":folder which contains the html/js assets.
+        """
+        from robotframework_ls.constants import (
+            DEFAULT_ROBOT_FLOW_EXPLORER_HTML_TEMPLATE,
+            DEFAULT_ROBOT_FLOW_EXPLORER_OPTIONS,
+        )
+        from robocorp_ls_core.uris import from_fs_path
+        from robotframework_ls.impl.robot_lsp_constants import (
+            OPTION_ROBOT_FLOW_EXPLORER_THEME_DARK,
+        )
+        from robotframework_ls.impl.robot_generated_lsp_constants import (
+            OPTION_ROBOT_FLOW_EXPLORER_THEME,
+        )
+
+        current_doc_uri = opts["currentFileUri"]
+        html_bundle_flow_folder_path = opts["htmlBundleFolderPath"]
+        log.info(
+            "Dispatched flow explorer internal command with args:",
+            str(current_doc_uri),
+            str(html_bundle_flow_folder_path),
+        )
+
+        def finish_and_convert_result(result: ActionResultDict) -> ActionResultDict:
+            log.info("Generating web page for visualization...")
+
+            if not result["success"]:
+                return result
+
+            # identify the selected theme & construct options
+            rfe_theme = self._config.get_setting(
+                OPTION_ROBOT_FLOW_EXPLORER_THEME,
+                str,
+                OPTION_ROBOT_FLOW_EXPLORER_THEME_DARK,
+            )
+            rfe_options = DEFAULT_ROBOT_FLOW_EXPLORER_OPTIONS.copy()
+            rfe_options["theme"] = rfe_theme
+            model = result["result"]
+
+            # filling in the HTML template
+            # adding the new model as data to Flow Explorer
+            replacement_html = DEFAULT_ROBOT_FLOW_EXPLORER_HTML_TEMPLATE.substitute(
+                rfe_options=json.dumps(rfe_options),
+                rfe_data=json.dumps(model),
+                rfe_favicon_path=Path(
+                    os.path.join(html_bundle_flow_folder_path, "favicon.png")
+                ).as_uri(),
+                rfe_js_path=Path(
+                    os.path.join(
+                        html_bundle_flow_folder_path, "robot_flow_explorer_bundle.js"
+                    )
+                ).as_uri(),
+            )
+            # create the temporary HTML file to display
+            with tempfile.NamedTemporaryFile(
+                mode="w", delete=False, suffix=".html"
+            ) as temp_file:
+                Path(temp_file.name).write_text(replacement_html, encoding="utf-8")
+                return {
+                    "result": from_fs_path(temp_file.name),
+                    "success": True,
+                    "message": None,
+                }
+
+        return self.async_api_forward(
+            "request_flow_explorer_model",
+            "api",
+            doc_uri=current_doc_uri,
+            uri=current_doc_uri,
+            __add_doc_uri_in_args__=False,
+            __convert_result__=finish_and_convert_result,
+        )
+
     @command_dispatcher("robot.listTests")
     def _list_tests(self, *arguments):
         doc_uri = arguments[0]["uri"]
@@ -727,7 +866,10 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
     @overrides(PythonLanguageServer.m_workspace__did_change_configuration)
     @log_and_silence_errors(log)
     def m_workspace__did_change_configuration(self, **kwargs) -> None:
+        from robotframework_ls.impl import robot_localization
+
         PythonLanguageServer.m_workspace__did_change_configuration(self, **kwargs)
+        robot_localization.set_global_from_config(self.config)
         self._server_manager.set_config(self.config)
 
     # --- Methods to forward to the api
@@ -885,6 +1027,7 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
         monitor: IMonitor,
         __timeout__=DEFAULT_COMPLETIONS_TIMEOUT,
         __log__=False,
+        __convert_result__=None,
         **kwargs,
     ):
         from robocorp_ls_core.client_base import wait_for_message_matcher
@@ -923,6 +1066,8 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
                 if __log__:
                     log.info("Result: %s", result)
                 if result:
+                    if __convert_result__ is not None:
+                        result = __convert_result__(result)
                     return result
 
                 error = msg.get("error")
@@ -937,6 +1082,8 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
         rf_api_client: IRobotFrameworkApiClient,
         request_method_name: str,
         monitor: Optional[IMonitor],
+        __timeout__=DEFAULT_COMPLETIONS_TIMEOUT,
+        __convert_result__=None,
         **kwargs,
     ):
         from robocorp_ls_core.client_base import wait_for_message_matcher
@@ -952,13 +1099,15 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
         if wait_for_message_matcher(
             message_matcher,
             rf_api_client.request_cancel,
-            DEFAULT_COMPLETIONS_TIMEOUT,
+            __timeout__,
             monitor,
         ):
             msg = message_matcher.msg
             if msg is not None:
                 result = msg.get("result")
                 if result:
+                    if __convert_result__ is not None:
+                        result = __convert_result__(result)
                     return result
 
         return None
