@@ -1,12 +1,13 @@
 from robocorp_ls_core.python_ls import PythonLanguageServer
 from robocorp_ls_core.basic import overrides
-from robocorp_ls_core.robotframework_log import get_logger
+from robocorp_ls_core.robotframework_log import get_logger, get_log_level
 from typing import Optional, List, Dict, Deque, Tuple, Sequence
 from robocorp_ls_core.protocols import (
     IConfig,
     IMonitor,
     ITestInfoTypedDict,
     IWorkspace,
+    ActionResultDict,
 )
 from functools import partial
 from robocorp_ls_core.jsonrpc.endpoint import require_monitor
@@ -38,6 +39,7 @@ import typing
 import sys
 import threading
 from robocorp_ls_core.jsonrpc.exceptions import JsonRpcException
+import os
 
 
 log = get_logger(__name__)
@@ -215,10 +217,13 @@ class RobotFrameworkServerApi(PythonLanguageServer):
         from robotframework_ls.impl.robot_lsp_constants import (
             OPTION_ROBOT_VARIABLES_LOAD_FROM_ARGUMENTS_FILE,
         )
+        from robotframework_ls.impl import robot_localization
 
         PythonLanguageServer.m_workspace__did_change_configuration(self, **kwargs)
         self.libspec_manager.config = self.config
 
+        robot_localization.set_global_from_config(self.config)
+
         try:
             variables_from_arguments_files = self.config.get_setting(
                 OPTION_ROBOT_VARIABLES_LOAD_FROM_ARGUMENTS_FILE, str, ""
@@ -429,11 +434,13 @@ class RobotFrameworkServerApi(PythonLanguageServer):
         if completion_context is None:
             return []
 
-        from robotframework_ls.impl import snippets_completions
+        from robotframework_ls.impl import snippets_completions, section_completions
 
         completions = snippets_completions.complete(completion_context)
         monitor.check_cancelled()
         completions.extend(self._complete_from_completion_context(completion_context))
+        completions.extend(section_completions.complete(completion_context))
+
         return completions
 
     def _complete_from_completion_context(self, completion_context):
@@ -503,6 +510,59 @@ class RobotFrameworkServerApi(PythonLanguageServer):
 
         return prepare_rename(completion_context)
 
+    def m_flow_explorer_model(self, uri):
+        func = partial(self._threaded_flow_explorer_model, uri)
+        func = require_monitor(func)
+        return func
+
+    def _threaded_flow_explorer_model(self, uri, monitor) -> ActionResultDict:
+        import json
+        from robocorp_ls_core import uris
+
+        # Note: it may actually be a directory (in which case we have to
+        # collect the robots inside it).
+        target_path = uris.to_fs_path(uri)
+
+        completion_contexts: List[ICompletionContext] = []
+        if not os.path.exists(target_path) or not os.path.isdir(target_path):
+            completion_context = self._create_completion_context(uri, 0, 0, monitor)
+            if completion_context is None:
+                # It's a single file and we don't have a context.
+                return {
+                    "success": False,
+                    "message": f"File: {target_path} does not exist.",
+                    "result": None,
+                }
+            completion_contexts.append(completion_context)
+
+        else:
+            # We're dealing with a directory.
+            for f in os.listdir(target_path):
+                if f.endswith(".robot"):
+                    f_uri = uris.from_fs_path(os.path.join(target_path, f))
+                    completion_context = self._create_completion_context(
+                        f_uri, 0, 0, monitor
+                    )
+                    if completion_context is None:
+                        continue
+                    completion_contexts.append(completion_context)
+
+            if not completion_contexts:
+                return {
+                    "success": False,
+                    "message": f"Unable to load any .robot files from: '{target_path}'.",
+                    "result": None,
+                }
+
+        from robotframework_ls.impl.flow_explorer_model_builder import (
+            build_flow_explorer_model,
+        )
+
+        model = build_flow_explorer_model(completion_contexts)
+        if get_log_level() >= 2:
+            log.debug("Model:", json.dumps(model))
+        return {"success": True, "message": None, "result": model}
+
     def m_find_definition(self, doc_uri, line, col):
         func = partial(self._threaded_find_definition, doc_uri, line, col)
         func = require_monitor(func)
@@ -679,8 +739,8 @@ class RobotFrameworkServerApi(PythonLanguageServer):
 
             try:
                 new_contents = robot_tidy_source_format(ast, dirname)
-            except ImportError:
-                log.critical(
+            except (ImportError, AttributeError):
+                log.exception(
                     "Unable to code-format because robotidy could not be imported."
                 )
                 return []
@@ -835,6 +895,61 @@ class RobotFrameworkServerApi(PythonLanguageServer):
 
         return list_tests(completion_context)
 
+    def m_collect_robot_documentation(
+        self,
+        doc_uri: str,
+        library_name: Optional[str] = None,
+        line: Optional[int] = None,
+        col: Optional[int] = None,
+    ):
+        func = partial(
+            self._threaded_collect_robot_documentation,
+            doc_uri,
+            library_name,
+            line,
+            col,
+        )
+        func = require_monitor(func)
+        return func
+
+    def _threaded_collect_robot_documentation(
+        self,
+        doc_uri: str,
+        library_name: Optional[str],
+        line: Optional[int],
+        col: Optional[int],
+        monitor: IMonitor,
+    ):
+        from robotframework_ls.impl.collect_robot_documentation import (
+            collect_robot_documentation,
+        )
+
+        if line is None:
+            line = 0
+        if col is None:
+            col = 0
+
+        ctx: Optional[ICompletionContext] = self._create_completion_context(
+            doc_uri, line, col, monitor
+        )
+        if ctx is None:
+            return {
+                "success": False,
+                "message": "Unable to create context to generate lib docs.",
+                "result": None,
+            }
+
+        try:
+            return collect_robot_documentation(library_name, ctx)
+        except Exception as e:
+            msg = f"Error collecting documentation: {str(e)}"
+            log.exception(msg)
+            return {
+                "success": False,
+                "message": msg,
+                "result": None,
+            }
+
     def m_evaluatable_expression(self, doc_uri: str, position: PositionTypedDict):
         func = partial(self._threaded_evaluatable_expression, doc_uri, position)
         func = require_monitor(func)
