@@ -112,6 +112,9 @@ class _ServerApi(object):
         self._config.update(config.get_full_settings())
         self._check_reinitialize_and_forward_settings_if_needed()
 
+    def get_interpreter_info(self) -> Optional[IInterpreterInfo]:
+        return self._interpreter_info
+
     def set_interpreter_info(self, interpreter_info: IInterpreterInfo) -> None:
         from robotframework_ls.config_extension import apply_interpreter_info_to_config
 
@@ -290,7 +293,11 @@ class _ServerApi(object):
                 def on_received_message(msg):
                     method = msg.get("method")
 
-                    if method in ("$/customProgress", "$/testsCollected"):
+                    if method in (
+                        "$/customProgress",
+                        "$/testsCollected",
+                        "window/showMessage",
+                    ):
                         robot_framework_language_server = language_server_ref()
                         if robot_framework_language_server is not None:
                             robot_framework_language_server.forward_msg(msg)
