@@ -4,15 +4,30 @@ Lengths checkers
 import re
 
 from robot.parsing.model.blocks import CommentSection, TestCase
-from robot.parsing.model.statements import Arguments, Comment, EmptyLine, KeywordCall
+from robot.parsing.model.statements import (
+    Arguments,
+    Comment,
+    Documentation,
+    EmptyLine,
+    KeywordCall,
+    Template,
+    TemplateArguments,
+)
+
+try:
+    from robot.api.parsing import Break, Continue, ReturnStatement
+except ImportError:
+    ReturnStatement, Break, Continue = None, None, None
 
 from robocop.checkers import RawFileChecker, VisitorChecker
-from robocop.rules import Rule, RuleParam, RuleSeverity
-from robocop.utils import get_section_name, last_non_empty_line, normalize_robot_name, pattern_type
+from robocop.rules import Rule, RuleParam, RuleSeverity, SeverityThreshold
+from robocop.utils import get_section_name, last_non_empty_line, normalize_robot_name, pattern_type, str2bool
 
 rules = {
     "0501": Rule(
         RuleParam(name="max_len", default=40, converter=int, desc="number of lines allowed in a keyword"),
+        RuleParam(name="ignore_docs", default=False, converter=str2bool, desc="Ignore documentation"),
+        SeverityThreshold("max_len", compare_method="greater"),
         rule_id="0501",
         name="too-long-keyword",
         msg="Keyword '{{ keyword_name }}' is too long ({{ keyword_length }}/{{ allowed_length}})",
@@ -20,6 +35,7 @@ rules = {
     ),
     "0502": Rule(
         RuleParam(name="min_calls", default=1, converter=int, desc="number of keyword calls required in a keyword"),
+        SeverityThreshold("min_calls", compare_method="less"),
         rule_id="0502",
         name="too-few-calls-in-keyword",
         msg="Keyword '{{ keyword_name }}' has too few keywords inside ({{ keyword_count }}/{{ min_allowed_count }})",
@@ -27,6 +43,7 @@ rules = {
     ),
     "0503": Rule(
         RuleParam(name="max_calls", default=10, converter=int, desc="number of keyword calls allowed in a keyword"),
+        SeverityThreshold("max_calls", compare_method="greater"),
         rule_id="0503",
         name="too-many-calls-in-keyword",
         msg="Keyword '{{ keyword_name }}' has too many keywords inside ({{ keyword_count }}/{{ max_allowed_count }})",
@@ -34,6 +51,8 @@ rules = {
     ),
     "0504": Rule(
         RuleParam(name="max_len", default=20, converter=int, desc="number of lines allowed in a test case"),
+        RuleParam(name="ignore_docs", default=False, converter=str2bool, desc="Ignore documentation"),
+        SeverityThreshold("max_len", compare_method="greater"),
         rule_id="0504",
         name="too-long-test-case",
         msg="Test case '{{ test_name }}' is too long ({{ test_length }}/{{ allowed_length }})",
@@ -41,13 +60,17 @@ rules = {
     ),
     "0505": Rule(
         RuleParam(name="max_calls", default=10, converter=int, desc="number of keyword calls allowed in a test case"),
+        RuleParam(name="ignore_templated", default=False, converter=str2bool, desc="Ignore templated tests"),
+        SeverityThreshold("max_calls", compare_method="greater"),
         rule_id="0505",
         name="too-many-calls-in-test-case",
         msg="Test case '{{ test_name }}' has too many keywords inside ({{ keyword_count }}/{{ max_allowed_count }})",
+        docs="Redesign the test and move complex logic to separate keywords to increase readiblity.",
         severity=RuleSeverity.WARNING,
     ),
     "0506": Rule(
         RuleParam(name="max_lines", default=400, converter=int, desc="number of lines allowed in a file"),
+        SeverityThreshold("max_lines", compare_method="greater"),
         rule_id="0506",
         name="file-too-long",
         msg="File has too many lines ({{ lines_count }}/{{max_allowed_count }})",
@@ -55,6 +78,7 @@ rules = {
     ),
     "0507": Rule(
         RuleParam(name="max_args", default=5, converter=int, desc="number of lines allowed in a file"),
+        SeverityThreshold("max_args", compare_method="greater"),
         rule_id="0507",
         name="too-many-arguments",
         msg="Keyword '{{ keyword_name }}' has too many arguments ({{ arguments_count }}/{{ max_allowed_count }})",
@@ -68,6 +92,7 @@ rules = {
             converter=pattern_type,
             desc="ignore lines that contain configured pattern",
         ),
+        SeverityThreshold("line_length"),
         rule_id="0508",
         name="line-too-long",
         msg="Line is too long ({{ line_length }}/{{ allowed_length }})",
@@ -77,7 +102,7 @@ rules = {
         
             robocop --configure line-too-long:ignore_pattern:pattern
         
-        The default pattern is `https?://\S+` that ignores the lines that look like an URL.
+        The default pattern is ``https?://\S+`` that ignores the lines that look like an URL.
 
         """,
     ),
@@ -88,6 +113,7 @@ rules = {
         RuleParam(
             name="max_returns", default=4, converter=int, desc="allowed number of returned values from a keyword"
         ),
+        SeverityThreshold("max_returns", compare_method="greater"),
         rule_id="0510",
         name="number-of-returned-values",
         msg="Too many return values ({{ return_count }}/{{ max_allowed_count }})",
@@ -172,25 +198,58 @@ rules = {
             converter=int,
             desc="number of test cases allowed in a templated suite",
         ),
+        SeverityThreshold("max_testcases or max_templated_testcases"),
         rule_id="0527",
         name="too-many-test-cases",
         msg="Too many test cases ({{ test_count }}/{{ max_allowed_count }})",
         severity=RuleSeverity.WARNING,
     ),
+    "0528": Rule(
+        RuleParam(name="min_calls", default=1, converter=int, desc="number of keyword calls required in a test case"),
+        RuleParam(name="ignore_templated", default=False, converter=str2bool, desc="Ignore templated tests"),
+        rule_id="0528",
+        name="too-few-calls-in-test-case",
+        msg="Test case '{{ test_name }}' has too few keywords inside ({{ keyword_count }}/{{ min_allowed_count }})",
+        docs="""
+        Test without keywords will fail. Add more keywords or set results using Fail, Pass, Skip keywords::
+        
+            *** Test Cases ***
+            Test case
+                [Tags]    smoke
+                Skip    Test case draft
+
+        """,
+        severity=RuleSeverity.ERROR,
+    ),
 }
 
 
+def check_node_length(node, ignore_docs):
+    if ignore_docs:
+        return node.end_lineno - node.lineno - get_documentation_length(node)
+    return node.end_lineno - node.lineno
+
+
+def get_documentation_length(node):
+    doc_len = 0
+    for child in node.body:
+        if isinstance(child, Documentation):
+            doc_len += child.end_lineno - child.lineno + 1
+    return doc_len
+
+
 class LengthChecker(VisitorChecker):
     """Checker for max and min length of keyword or test case. It analyses number of lines and also number of
     keyword calls (as you can have just few keywords but very long ones or vice versa).
     """
 
     reports = (
-        "too-long-keyword",
         "too-few-calls-in-keyword",
+        "too-few-calls-in-test-case",
         "too-many-calls-in-keyword",
-        "too-long-test-case",
         "too-many-calls-in-test-case",
+        "too-long-keyword",
+        "too-long-test-case",
         "file-too-long",
         "too-many-arguments",
     )
@@ -203,6 +262,7 @@ class LengthChecker(VisitorChecker):
                 max_allowed_count=self.param("file-too-long", "max_lines"),
                 node=node,
                 lineno=node.end_lineno,
+                sev_threshold_value=node.end_lineno,
             )
         super().visit_File(node)
 
@@ -219,9 +279,10 @@ class LengthChecker(VisitorChecker):
                         arguments_count=args_number,
                         max_allowed_count=self.param("too-many-arguments", "max_args"),
                         node=node,
+                        sev_threshold_value=args_number,
                     )
                 break
-        length = LengthChecker.check_node_length(node)
+        length = check_node_length(node, ignore_docs=self.param("too-long-keyword", "ignore_docs"))
         if length > self.param("too-long-keyword", "max_len"):
             self.report(
                 "too-long-keyword",
@@ -231,6 +292,7 @@ class LengthChecker(VisitorChecker):
                 node=node,
                 lineno=node.end_lineno,
                 ext_disablers=(node.lineno, last_non_empty_line(node)),
+                sev_threshold_value=length,
             )
             return
         key_calls = LengthChecker.count_keyword_calls(node)
@@ -241,20 +303,32 @@ class LengthChecker(VisitorChecker):
                 keyword_count=key_calls,
                 min_allowed_count=self.param("too-few-calls-in-keyword", "min_calls"),
                 node=node,
+                end_col=node.col_offset + len(node.name) + 1,
+                sev_threshold_value=key_calls,
             )
-            return
-        if key_calls > self.param("too-many-calls-in-keyword", "max_calls"):
+        elif key_calls > self.param("too-many-calls-in-keyword", "max_calls"):
             self.report(
                 "too-many-calls-in-keyword",
                 keyword_name=node.name,
                 keyword_count=key_calls,
                 max_allowed_count=self.param("too-many-calls-in-keyword", "max_calls"),
                 node=node,
+                end_col=node.col_offset + len(node.name) + 1,
+                sev_threshold_value=key_calls,
             )
-            return
+
+    def test_is_templated(self, node):
+        if self.templated_suite:
+            return True
+        if not node.body:
+            return False
+        for statement in node.body:
+            if isinstance(statement, Template):
+                return True
+        return False
 
     def visit_TestCase(self, node):  # noqa
-        length = LengthChecker.check_node_length(node)
+        length = check_node_length(node, ignore_docs=self.param("too-long-test-case", "ignore_docs"))
         if length > self.param("too-long-test-case", "max_len"):
             self.report(
                 "too-long-test-case",
@@ -262,29 +336,50 @@ class LengthChecker(VisitorChecker):
                 test_length=length,
                 allowed_length=self.param("too-long-test-case", "max_len"),
                 node=node,
+                sev_threshold_value=length,
             )
+        test_is_templated = self.test_is_templated(node)
+        skip_too_many = test_is_templated and self.param("too-many-calls-in-test-case", "ignore_templated")
+        skip_too_few = test_is_templated and self.param("too-few-calls-in-test-case", "ignore_templated")
+        if skip_too_few and skip_too_many:
+            return
         key_calls = LengthChecker.count_keyword_calls(node)
-        if key_calls > self.param("too-many-calls-in-test-case", "max_calls"):
+        if not skip_too_many and (key_calls > self.param("too-many-calls-in-test-case", "max_calls")):
             self.report(
                 "too-many-calls-in-test-case",
                 test_name=node.name,
                 keyword_count=key_calls,
                 max_allowed_count=self.param("too-many-calls-in-test-case", "max_calls"),
                 node=node,
+                sev_threshold_value=key_calls,
+                end_col=node.col_offset + len(node.name) + 1,
+            )
+        elif not skip_too_few and (key_calls < self.param("too-few-calls-in-test-case", "min_calls")):
+            self.report(
+                "too-few-calls-in-test-case",
+                test_name=node.name,
+                keyword_count=key_calls,
+                min_allowed_count=self.param("too-few-calls-in-test-case", "min_calls"),
+                node=node,
+                sev_threshold_value=key_calls,
+                end_col=node.col_offset + len(node.name) + 1,
             )
-            return
-
-    @staticmethod
-    def check_node_length(node):
-        return node.end_lineno - node.lineno
 
     @staticmethod
     def count_keyword_calls(node):
-        if isinstance(node, KeywordCall):
+        # ReturnStatement is imported and evaluates to true in RF 5.0+, we don't need to also check Break/Continue
+        if isinstance(node, (KeywordCall, TemplateArguments)) or ReturnStatement and isinstance(node, (Break, Continue, ReturnStatement)):
             return 1
         if not hasattr(node, "body"):
             return 0
-        return sum(LengthChecker.count_keyword_calls(child) for child in node.body)
+        calls = sum(LengthChecker.count_keyword_calls(child) for child in node.body)
+        while node and getattr(node, "orelse", None):
+            node = node.orelse
+            calls += sum(LengthChecker.count_keyword_calls(child) for child in node.body)
+        while node and getattr(node, "next", None):
+            node = node.next
+            calls += sum(LengthChecker.count_keyword_calls(child) for child in node.body)
+        return calls
 
 
 class LineLengthChecker(RawFileChecker):
@@ -305,6 +400,7 @@ class LineLengthChecker(RawFileChecker):
                 line_length=len(line),
                 allowed_length=self.param("line-too-long", "line_length"),
                 lineno=lineno,
+                sev_threshold_value=len(line),
             )
 
 
@@ -352,6 +448,7 @@ class NumberOfReturnedArgsChecker(VisitorChecker):
                 max_allowed_count=self.param("number-of-returned-values", "max_returns"),
                 node=node,
                 col=node.data_tokens[0].col_offset + 1,
+                sev_threshold_value=return_count,
             )
 
 
@@ -478,5 +575,9 @@ class TestCaseNumberChecker(VisitorChecker):
         discovered_testcases = sum([isinstance(child, TestCase) for child in node.body])
         if discovered_testcases > max_testcases:
             self.report(
-                "too-many-test-cases", test_count=discovered_testcases, max_allowed_count=max_testcases, node=node
+                "too-many-test-cases",
+                test_count=discovered_testcases,
+                max_allowed_count=max_testcases,
+                node=node,
+                sev_threshold_value=discovered_testcases,
             )
