@@ -21,6 +21,7 @@ from robocop.utils import (
     remove_robot_vars,
     token_col,
 )
+from robocop.utils.run_keywords import iterate_keyword_names
 
 rules = {
     "0301": Rule(
@@ -290,6 +291,31 @@ rules = {
         reports any occurrence of `@[` characters.
         """,
     ),
+    "0321": Rule(
+        rule_id="0321",
+        name="deprecated-with-name",
+        msg=(
+            "'WITH NAME' alias marker is deprecated since Robot Framework 5.1 version "
+            "and will be removed in the future release. Use 'AS' instead"
+        ),
+        severity=RuleSeverity.WARNING,
+        version=">=5.1",
+        docs="""
+        ``WITH NAME`` marker that is used when giving an alias to an imported library is going to be renamed to ``AS``.
+        The motivation is to be consistent with Python that uses ``as`` for similar purpose.
+        
+        Code with the deprecated marker::
+        
+            *** Settings ***
+            Library    Collections    WITH NAME    AliasedName
+        
+        Code with the supported marker::
+        
+            *** Settings ***
+            Library    Collections    AS    AliasedName
+        
+        """,
+    ),
 }
 
 
@@ -307,7 +333,9 @@ class InvalidCharactersInNameChecker(VisitorChecker):
             suite_name = Path(source).stem
             if "__init__" in suite_name:
                 suite_name = Path(source).parent.name
-            for iter in self.param("not-allowed-char-in-filename", "pattern").finditer(suite_name):
+            for iter in self.param("not-allowed-char-in-filename", "pattern").finditer(
+                suite_name
+            ):
                 self.report(
                     "not-allowed-char-in-filename",
                     character=iter.group(),
@@ -318,7 +346,7 @@ class InvalidCharactersInNameChecker(VisitorChecker):
         super().visit_File(node)
 
     def check_if_pattern_in_node_name(self, node, name_of_node, is_keyword=False):
-        """ Search if regex pattern found from node name.
+        """Search if regex pattern found from node name.
         Skips embedded variables from keyword name
         """
         node_name = node.name
@@ -332,8 +360,10 @@ class InvalidCharactersInNameChecker(VisitorChecker):
             #   1. `Keyword With `
             #   2. ` Two `
             #   3. ` Argument` - last part is searched in finditer part after this loop
-            tmp_node_name = node_name[start_pos:variable[0]]
-            match = self.param("not-allowed-char-in-name", "pattern").search(tmp_node_name)
+            tmp_node_name = node_name[start_pos : variable[0]]
+            match = self.param("not-allowed-char-in-name", "pattern").search(
+                tmp_node_name
+            )
             if match:
                 self.report(
                     "not-allowed-char-in-name",
@@ -344,7 +374,9 @@ class InvalidCharactersInNameChecker(VisitorChecker):
                 )
             start_pos = variable[1]
 
-        for iter in self.param("not-allowed-char-in-name", "pattern").finditer(node_name, start_pos):
+        for iter in self.param("not-allowed-char-in-name", "pattern").finditer(
+            node_name, start_pos
+        ):
             self.report(
                 "not-allowed-char-in-name",
                 character=iter.group(),
@@ -407,6 +439,19 @@ class KeywordNamingChecker(VisitorChecker):
             "except": reserved_error_msg("except", "'TRY EXCEPT'"),
             "finally": reserved_error_msg("finally", "'TRY EXCEPT'"),
         },
+        6: {
+            "if": uppercase_error_msg("if"),
+            "else if": uppercase_error_msg("else if"),
+            "else": uppercase_error_msg("else"),
+            "for": reserved_error_msg("for", "'FOR' loop"),
+            "end": reserved_error_msg("end", "'FOR', 'IF' or 'TRY EXCEPT'"),
+            "while": uppercase_error_msg("while"),
+            "continue": uppercase_error_msg("continue"),
+            "return": uppercase_error_msg("return"),
+            "try": reserved_error_msg("try", "'TRY EXCEPT'"),
+            "except": reserved_error_msg("except", "'TRY EXCEPT'"),
+            "finally": reserved_error_msg("finally", "'TRY EXCEPT'"),
+        },
     }
     else_statements = {"else", "else if"}
     bdd = {"given", "when", "and", "but", "then"}
@@ -416,10 +461,17 @@ class KeywordNamingChecker(VisitorChecker):
         self.inside_if_block = False
         super().__init__()
 
-    def visit_Setup(self, node):  # noqa
-        self.check_keyword_naming(node.name, node)
+    def check_keyword_naming_with_subkeywords(self, node, name_token_type):
+        for keyword in iterate_keyword_names(node, name_token_type):
+            self.check_keyword_naming(keyword.value, keyword)
 
-    visit_TestTeardown = visit_SuiteTeardown = visit_Teardown = visit_TestSetup = visit_SuiteSetup = visit_Setup
+    def visit_Setup(self, node):  # noqa
+        self.check_bdd_keywords(node.name, node)
+        self.check_keyword_naming_with_subkeywords(node, Token.NAME)
+
+    visit_TestTeardown = (
+        visit_SuiteTeardown
+    ) = visit_Teardown = visit_TestSetup = visit_SuiteSetup = visit_Setup
 
     def visit_Keyword(self, node):  # noqa
         if not node.name:
@@ -429,9 +481,14 @@ class KeywordNamingChecker(VisitorChecker):
         self.generic_visit(node)
 
     def visit_KeywordCall(self, node):  # noqa
-        if self.inside_if_block and node.keyword and node.keyword.lower() in self.else_statements:
+        if (
+            self.inside_if_block
+            and node.keyword
+            and node.keyword.lower() in self.else_statements
+        ):
             self.report("else-not-upper-case", node=node, col=keyword_col(node))
-        self.check_keyword_naming(node.keyword, node)
+        self.check_keyword_naming_with_subkeywords(node, Token.KEYWORD)
+        self.check_bdd_keywords(node.keyword, node)
 
     def visit_If(self, node):  # noqa
         self.inside_if_block = True
@@ -443,41 +500,56 @@ class KeywordNamingChecker(VisitorChecker):
             return
         if keyword_name == r"/":  # old for loop, / are interpreted as keywords
             return
-        if (
-            isinstance(node, KeywordCall)
-            and normalize_robot_name(keyword_name, remove_prefix="builtin.") == "runkeywordif"
-        ):
-            for token in node.data_tokens:
-                if (token.value.lower() in self.else_statements) and not token.value.isupper():
-                    self.report(
-                        "keyword-name-is-reserved-word",
-                        keyword_name=token.value,
-                        error_msg=reserved_error_msg(token.value, "'Run Keyword If'"),
-                        node=node,
-                        col=token.col_offset + 1,
-                    )
-        elif self.check_if_keyword_is_reserved(keyword_name, node):
+        if self.check_if_keyword_is_reserved(keyword_name, node):
             return
-        self.check_bdd_keywords(keyword_name, node)
         normalized = remove_robot_vars(keyword_name)
-        normalized = self.param("wrong-case-in-keyword-name", "pattern").sub("", normalized)
-        normalized = normalized.split(".")[-1]  # remove any imports ie ExternalLib.SubLib.Log -> Log
+        normalized = self.param("wrong-case-in-keyword-name", "pattern").sub(
+            "", normalized
+        )
+        normalized = normalized.split(".")[
+            -1
+        ]  # remove any imports ie ExternalLib.SubLib.Log -> Log
         normalized = normalized.replace("'", "")  # replace ' apostrophes
         if "_" in normalized:
-            self.report("underscore-in-keyword-name", keyword_name=keyword_name, node=node)
+            self.report(
+                "underscore-in-keyword-name",
+                keyword_name=keyword_name,
+                node=node,
+                col=node.col_offset + 1,
+                end_col=node.end_col_offset + 1,
+            )
         words = self.letter_pattern.sub(" ", normalized).split(" ")
-        if self.param("wrong-case-in-keyword-name", "convention") == "first_word_capitalized":
+        if (
+            self.param("wrong-case-in-keyword-name", "convention")
+            == "first_word_capitalized"
+        ):
             words = words[:1]
         if any(word[0].islower() for word in words if word):
-            self.report("wrong-case-in-keyword-name", keyword_name=keyword_name, node=node)
+            self.report(
+                "wrong-case-in-keyword-name",
+                keyword_name=keyword_name,
+                node=node,
+                col=node.col_offset + 1,
+                end_col=node.end_col_offset + 1,
+            )
 
     def check_bdd_keywords(self, keyword_name, node):
-        if keyword_name.lower() not in self.bdd or isinstance(node, Keyword):
+        if not keyword_name or keyword_name.lower() not in self.bdd:
             return
         arg = node.get_token(Token.ARGUMENT)
-        suffix = f". Use one space between: '{keyword_name.title()} {arg.value}'" if arg else ""
+        suffix = (
+            f". Use one space between: '{keyword_name.title()} {arg.value}'"
+            if arg
+            else ""
+        )
         col = token_col(node, Token.NAME, Token.KEYWORD)
-        self.report("bdd-without-keyword-call", keyword_name=keyword_name, error_msg=suffix, node=node, col=col)
+        self.report(
+            "bdd-without-keyword-call",
+            keyword_name=keyword_name,
+            error_msg=suffix,
+            node=node,
+            col=col,
+        )
 
     def check_if_keyword_is_reserved(self, keyword_name, node):
         # if there is typo in syntax, it is interpreted as keyword
@@ -487,13 +559,13 @@ class KeywordNamingChecker(VisitorChecker):
         if keyword_name.lower() in self.else_statements and self.inside_if_block:
             return False  # handled by else-not-upper-case
         error_msg = reserved[keyword_name.lower()]
-        col = keyword_col(node) if isinstance(node, KeywordCall) else keyword_col(node.header)
         self.report(
             "keyword-name-is-reserved-word",
             keyword_name=keyword_name,
             error_msg=error_msg,
             node=node,
-            col=col,
+            col=node.col_offset + 1,
+            end_col=node.end_col_offset + 1,
         )
         return True
 
@@ -507,6 +579,13 @@ class SettingsNamingChecker(VisitorChecker):
         "empty-library-alias",
         "duplicated-library-alias",
     )
+    ALIAS_TOKENS = (
+        [Token.WITH_NAME] if ROBOT_VERSION.major < 5 else [Token.WITH_NAME, "AS"]
+    )
+    # Separating alias values since RF 3 uses WITH_NAME instead of WITH NAME
+    ALIAS_TOKENS_VALUES = (
+        ["WITH NAME"] if ROBOT_VERSION.major < 5 else [Token.WITH_NAME, "AS"]
+    )
 
     def __init__(self):
         self.section_name_pattern = re.compile(r"\*\*\*\s.+\s\*\*\*")
@@ -514,10 +593,15 @@ class SettingsNamingChecker(VisitorChecker):
 
     def visit_SectionHeader(self, node):  # noqa
         name = node.data_tokens[0].value
-        if not self.section_name_pattern.match(name) or not (name.istitle() or name.isupper()):
+        if not self.section_name_pattern.match(name) or not (
+            name.istitle() or name.isupper()
+        ):
             valid_name = f"*** {node.name.title()} ***"
             self.report(
-                "section-name-invalid", section_title_case=valid_name, section_upper_case=valid_name.upper(), node=node
+                "section-name-invalid",
+                section_title_case=valid_name,
+                section_upper_case=valid_name.upper(),
+                node=node,
             )
 
     def visit_Setup(self, node):  # noqa
@@ -539,17 +623,23 @@ class SettingsNamingChecker(VisitorChecker):
         visit_ResourceImport
     ) = (
         visit_VariablesImport
-    ) = visit_Documentation = visit_Tags = visit_Timeout = visit_Template = visit_Arguments = visit_Return = visit_Setup
+    ) = (
+        visit_Documentation
+    ) = (
+        visit_Tags
+    ) = visit_Timeout = visit_Template = visit_Arguments = visit_Return = visit_Setup
 
     def visit_LibraryImport(self, node):  # noqa
         self.check_setting_name(node.data_tokens[0].value, node)
-        with_name = node.get_token(Token.WITH_NAME)
+        with_name = node.get_token(*self.ALIAS_TOKENS)
         if with_name is None:
             for arg in node.get_tokens(Token.ARGUMENT):
-                if arg.value and arg.value == "WITH NAME":
+                if arg.value and arg.value in self.ALIAS_TOKENS_VALUES:
                     self.report("empty-library-alias", node=arg, col=arg.col_offset + 1)
         else:
-            if node.alias.replace(" ", "") == node.name.replace(" ", ""):  # New Name == NewName
+            if node.alias.replace(" ", "") == node.name.replace(
+                " ", ""
+            ):  # New Name == NewName
                 name_token = node.get_tokens(Token.NAME)[-1]
                 self.report(
                     "duplicated-library-alias",
@@ -574,7 +664,9 @@ class TestCaseNamingChecker(VisitorChecker):
         if not node.name:
             self.report("test-case-name-is-empty", node=node)
         elif not node.name[0].isupper():
-            self.report("not-capitalized-test-case-title", test_name=node.name, node=node)
+            self.report(
+                "not-capitalized-test-case-title", test_name=node.name, node=node
+            )
 
 
 class VariableNamingChecker(VisitorChecker):
@@ -600,7 +692,11 @@ class VariableNamingChecker(VisitorChecker):
             if not child.data_tokens:
                 continue
             token = child.data_tokens[0]
-            if token.type == Token.VARIABLE and token.value and not token.value.isupper():
+            if (
+                token.type == Token.VARIABLE
+                and token.value
+                and not token.value.isupper()
+            ):
                 self.report(
                     "section-variable-not-uppercase",
                     variable_name=token.value,
@@ -620,7 +716,10 @@ class VariableNamingChecker(VisitorChecker):
 
         if not node.keyword:
             return
-        if normalize_robot_name(node.keyword, remove_prefix="builtin.") in self.set_variable_variants:
+        if (
+            normalize_robot_name(node.keyword, remove_prefix="builtin.")
+            in self.set_variable_variants
+        ):
             if len(node.data_tokens) < 2:
                 return
             token = node.data_tokens[1]
@@ -673,12 +772,17 @@ class SimilarVariableChecker(VisitorChecker):
             # read arguments from Test Case or Keyword
             if isinstance(child, Arguments):
                 for token in child.get_tokens(Token.ARGUMENT):
-                    self.variables[normalize_robot_var_name(token.value)].add(token.value)
+                    self.variables[normalize_robot_var_name(token.value)].add(
+                        token.value
+                    )
 
     def find_similar_variables(self, tokens, node):
         for token in tokens:
             normalized_token = normalize_robot_var_name(token.value)
-            if normalized_token in self.variables and token.value not in self.variables[normalized_token]:
+            if (
+                normalized_token in self.variables
+                and token.value not in self.variables[normalized_token]
+            ):
                 self.report(
                     "possible-variable-overwriting",
                     variable_name=token.value,
@@ -694,7 +798,7 @@ class SimilarVariableChecker(VisitorChecker):
 class DeprecatedStatementChecker(VisitorChecker):
     """Checker for deprecated statements."""
 
-    reports = ("deprecated-statement",)
+    reports = ("deprecated-statement", "deprecated-with-name")
     deprecated_keywords = {
         5: {
             "runkeywordunless": "IF",
@@ -714,7 +818,9 @@ class DeprecatedStatementChecker(VisitorChecker):
     def visit_SuiteSetup(self, node):  # noqa
         self.check_if_keyword_is_deprecated(node.name, node)
 
-    visit_TestSetup = visit_Setup = visit_SuiteTeardown = visit_TestTeardown = visit_Teardown = visit_SuiteSetup
+    visit_TestSetup = (
+        visit_Setup
+    ) = visit_SuiteTeardown = visit_TestTeardown = visit_Teardown = visit_SuiteSetup
 
     def visit_Return(self, node):  # noqa
         """For RETURN use visit_ReturnStatement - visit_Return will most likely visit RETURN in the future"""
@@ -730,7 +836,9 @@ class DeprecatedStatementChecker(VisitorChecker):
         )
 
     def check_if_keyword_is_deprecated(self, keyword_name, node):
-        normalized_keyword_name = normalize_robot_name(keyword_name, remove_prefix="builtin.")
+        normalized_keyword_name = normalize_robot_name(
+            keyword_name, remove_prefix="builtin."
+        )
         deprecated_statements = self.deprecated_keywords.get(ROBOT_VERSION.major, {})
         if normalized_keyword_name in deprecated_statements:
             alternative = deprecated_statements[normalized_keyword_name]
@@ -743,3 +851,17 @@ class DeprecatedStatementChecker(VisitorChecker):
                 col=col,
                 version=f"{ROBOT_VERSION.major}.*",
             )
+
+    def visit_LibraryImport(self, node):  # noqa
+        if ROBOT_VERSION.major < 5 or (
+            ROBOT_VERSION.major == 5 and ROBOT_VERSION.minor == 0
+        ):
+            return
+        with_name_token = node.get_token(Token.WITH_NAME)
+        if not with_name_token or with_name_token.value == "AS":
+            return
+        self.report(
+            "deprecated-with-name",
+            node=with_name_token,
+            col=with_name_token.col_offset + 1,
+        )
