@@ -1,12 +1,13 @@
 from typing import Optional
 
-from robot.api.parsing import EmptyLine, ModelTransformer, Token
+from robot.api.parsing import EmptyLine, Token
 
 from robotidy.disablers import skip_section_if_disabled
+from robotidy.transformers import Transformer
 from robotidy.utils import is_suite_templated
 
 
-class NormalizeNewLines(ModelTransformer):
+class NormalizeNewLines(Transformer):
     """
     Normalize new lines.
 
@@ -22,10 +23,10 @@ class NormalizeNewLines(ModelTransformer):
 
     If the suite contains Test Template tests will not be separated by empty lines unless ``separate_templated_tests``
     is set to True.
-
-    See https://robotidy.readthedocs.io/en/latest/transformers/NormalizeNewLines.html for more examples.
     """
 
+    WHITESPACE_TOKENS = {Token.EOL, Token.SEPARATOR}
+
     def __init__(
         self,
         test_case_lines: int = 1,
@@ -34,6 +35,7 @@ class NormalizeNewLines(ModelTransformer):
         separate_templated_tests: bool = False,
         consecutive_lines: int = 1,
     ):
+        super().__init__()
         self.test_case_lines = test_case_lines
         self.keyword_lines = keyword_lines if keyword_lines is not None else test_case_lines
         self.section_lines = section_lines
@@ -86,7 +88,10 @@ class NormalizeNewLines(ModelTransformer):
 
     def visit_Statement(self, node):  # noqa
         tokens = []
+        cont = node.get_token(Token.CONTINUATION)
         for line in node.lines:
+            if cont and all(token.type in self.WHITESPACE_TOKENS for token in line):
+                continue
             if line[-1].type == Token.EOL:
                 line[-1].value = "\n"  # TODO: use global formatting in the future
             tokens.extend(line)
