@@ -1,17 +1,18 @@
-from itertools import takewhile
-
-from robot.api.parsing import ModelTransformer, Token
+from robot.api.parsing import KeywordCall, Token
 
 try:
-    from robot.api.parsing import InlineIfHeader
+    from robot.api.parsing import InlineIfHeader, ReturnStatement
 except ImportError:
     InlineIfHeader = None
+    ReturnStatement = None
 
 from robotidy.disablers import skip_if_disabled, skip_section_if_disabled
 from robotidy.exceptions import InvalidParameterValueError
+from robotidy.skip import Skip
+from robotidy.transformers import Transformer
 
 
-class NormalizeSeparators(ModelTransformer):
+class NormalizeSeparators(Transformer):
     """
     Normalize separators and indents.
 
@@ -21,12 +22,15 @@ class NormalizeSeparators(ModelTransformer):
     You can decide which sections should be transformed by configuring
     ``sections = comments,settings,variables,keywords,testcases`` param.
 
-    Supports global formatting params: ``--startline`` and ``--endline``.
-
-    See https://robotidy.readthedocs.io/en/latest/transformers/NormalizeSeparators.html for more examples.
+    To not format documentation configure ``skip_documentation`` to ``True``.
     """
 
-    def __init__(self, sections: str = None):
+    HANDLES_SKIP = frozenset(
+        {"skip_documentation", "skip_keyword_call", "skip_keyword_call_pattern", "skip_comments", "skip_block_comments"}
+    )
+
+    def __init__(self, sections: str = None, skip: Skip = None):
+        super().__init__(skip=skip)
         self.indent = 0
         self.sections = self.parse_sections(sections)
         self.is_inline = False
@@ -113,17 +117,9 @@ class NormalizeSeparators(ModelTransformer):
             return node
         self.is_inline = InlineIfHeader and isinstance(node.header, InlineIfHeader)
         self.visit_Statement(node.header)
-        indent = 1
-        if self.is_inline:
-            indent = self.indent
-            self.indent = 1
-        else:
-            self.indent += 1
+        self.indent += 1
         node.body = [self.visit(item) for item in node.body]
-        if self.is_inline:
-            self.indent = indent
-        else:
-            self.indent -= 1
+        self.indent -= 1
         if node.orelse:
             self.visit(node.orelse)
         if node.end:
@@ -131,13 +127,37 @@ class NormalizeSeparators(ModelTransformer):
         self.is_inline = False
         return node
 
+    def visit_Documentation(self, doc):  # noqa
+        if self.skip.documentation:
+            has_pipes = doc.tokens[0].value.startswith("|")
+            return self._handle_spaces(doc, has_pipes, only_indent=True)
+        return self.visit_Statement(doc)
+
+    def visit_KeywordCall(self, keyword):  # noqa
+        if self.skip.keyword_call(keyword):
+            return keyword
+        return self.visit_Statement(keyword)
+
+    def visit_Comment(self, node):  # noqa
+        if self.skip.comment(node):
+            return node
+        return self.visit_Statement(node)
+
+    def is_keyword_inside_inline_if(self, node):
+        return self.is_inline and (
+            isinstance(node, KeywordCall) or ReturnStatement and isinstance(node, ReturnStatement)
+        )
+
     @skip_if_disabled
     def visit_Statement(self, statement):  # noqa
+        if statement is None:
+            return None
         has_pipes = statement.tokens[0].value.startswith("|")
         return self._handle_spaces(statement, has_pipes)
 
-    def _handle_spaces(self, statement, has_pipes):
+    def _handle_spaces(self, statement, has_pipes, only_indent=False):
         new_tokens = []
+        prev_token = None
         for line in statement.lines:
             prev_sep = False
             for index, token in enumerate(line):
@@ -145,10 +165,16 @@ class NormalizeSeparators(ModelTransformer):
                     if prev_sep:
                         continue
                     prev_sep = True
-                    count = self.indent if index == 0 else 1
-                    token.value = self.formatting_config.separator * count
+                    if index == 0 and not self.is_keyword_inside_inline_if(statement):
+                        token.value = self.formatting_config.indent * self.indent
+                    elif not only_indent:
+                        if prev_token and prev_token.type == Token.CONTINUATION:
+                            token.value = self.formatting_config.continuation_indent
+                        else:
+                            token.value = self.formatting_config.separator
                 else:
                     prev_sep = False
+                    prev_token = token
                 if has_pipes and index == len(line) - 2:
                     token.value = token.value.rstrip()
                 new_tokens.append(token)
